==> em_tasks-0.1.13-py2.py3-none-any.whl: em_tasks/utils/shipstation_orders_syncer.py <==
# -*- coding: utf-8 -*-

import datetime
import pdb

from shipstation.api import *

from em_celery import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class ShipstationOrdersSyncer():
  def __init__(self, shipstation_api_key, shipstation_api_password, days=14):
    self.shipstation_api_key = shipstation_api_key
    self.shipstation_api_password = shipstation_api_password
    self.client = ShipStation(key=shipstation_api_key, secret=shipstation_api_password)
    self.days = days

  def run(self):
    batch_size = 50

    today = datetime.datetime.utcnow()
    from_date = today - datetime.timedelta(days=self.days)
    ss_orders = dict()
    for order_item in OrderItem.select(OrderItem, FulfillOrder).join(FulfillOrder, on=(OrderItem.id == FulfillOrder.order_item_id), attr='fulfill_order').where((FulfillOrder.stc == 0) & (OrderItem.s_purchase_date > from_date)).execute():
      ss_order = StoreUtil.convert_order_item_to_shipstation_order(order_item)
      if not ss_order:
        continue

      ss_orders[ss_order.order_number] = ss_order

    for order_item in OrderItem.select().where((OrderItem.source == 'EMP') & (OrderItem.s_purchase_date > from_date)).execute():
      ss_order = StoreUtil.convert_order_item_to_shipstation_order(order_item)
      if not ss_order:
        continue

      ss_orders[ss_order.order_number] = ss_order

    to_update = []
    to_create = []

    order_numbers = list(ss_orders.keys())
    for so in ShipstationOrder.select().where(ShipstationOrder.order_number.in_(order_numbers)).execute():
      ss_order = ss_orders.pop(so.order_number, None)
      if ss_order.order_status != 'cancelled':
        continue

      ss_order.order_key = so.order_key
      to_update.append(ss_order)

    for _, ss_order in ss_orders.items():
      to_create.append(ss_order)

    if to_update:
      chunks = [to_update[x:x + batch_size] for x in range(0, len(to_update), batch_size)]
      for chunk in chunks:
        for so in chunk:
          self.client.add_order(so)
        try:
          resp = self.client.submit_orders()
          self.client.clear_orders()
          self.save_shipstation_orders(resp)
        except Exception as e:
          logger.exception(e)

    if to_create:
      chunks = [to_create[x:x + batch_size] for x in range(0, len(to_create), batch_size)]
      for chunk in chunks:
        for so in chunk:
          self.client.add_order(so)
        try:
          resp = self.client.submit_orders()
          self.client.clear_orders()
          self.save_shipstation_orders(resp)
        except Exception as e:
          logger.exception(e)

  def load_orders_from_shipstation(self):
    pass

  def save_shipstation_orders(self, resp):
    sos = dict()
    res = resp.json()
    for result in res['results']:
      succeed = result.get('success', True)
      if not succeed:
        continue

      order_number = result['orderNumber']
      sos[order_number] = {
        'order_id': result['orderId'],
        'order_number': result['orderNumber'],
        'order_key': result['orderKey']
      }

    order_numbers = list(sos.keys())
    for so in ShipstationOrder.select().where(ShipstationOrder.order_number.in_(order_numbers)).execute():
      if so.order_number not in sos:
        continue

      sso = sos.pop(so.order_number)
      if so.order_id != sso['order_id'] or so.order_key != sso['order_key']:
        so.order_id = sso['order_id']
        so.order_key = sso['order_key']
        so.save()

    if sos:
      s_order_numbers = []
      sos_mapping = dict()
      for _, so in sos.items():
        parts = so['order_number'].split('-')
        s_line_item_id = parts.pop()
        s_order_number = '-'.join(parts)
        s_order_numbers.append(s_order_number)
      for oi in OrderItem.select().where(OrderItem.s_order_number.in_(s_order_numbers)).execute():
        k = '{}-{}'.format(oi.s_order_number, oi.s_line_item_id)
        if k in sos:
          sos[k]['order_item_id'] = oi.id

      to_create = []
      for so in sos.values():
        if 'order_item_id' not in so:
          continue

        to_create.append(so)

      ShipstationOrder.insert_many(to_create).execute()

==> em_tasks-0.1.35-py2.py3-none-any.whl: em_tasks/utils/shipstation_orders_syncer.py <==
# -*- coding: utf-8 -*-

import datetime
import pdb

from shipstation.api import *

from em_celery import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class ShipstationOrdersSyncer():
  def __init__(self, shipstation_api_key, shipstation_api_password, store_id=None, days=14):
    self.shipstation_api_key = shipstation_api_key
    self.shipstation_api_password = shipstation_api_password
    self.client = ShipStation(key=shipstation_api_key, secret=shipstation_api_password)
    self.days = days
    self.store_id = store_id

  def run(self):
    batch_size = 50

    today = datetime.datetime.utcnow()
    from_date = today - datetime.timedelta(days=self.days)
    ss_orders = dict()
    for order_item in OrderItem.select(OrderItem, FulfillOrder).join(FulfillOrder, on=(OrderItem.id == FulfillOrder.order_item_id), attr='fulfill_order').where((FulfillOrder.stc == 0) & (OrderItem.s_purchase_date > from_date)).execute():
      ss_order = StoreUtil.convert_order_item_to_shipstation_order(order_item)
      if not ss_order:
        continue

      ss_orders[ss_order.order_number] = ss_order

    for order_item in OrderItem.select().where((OrderItem.source == 'EMP') & (OrderItem.s_purchase_date > from_date)).execute():
      ss_order = StoreUtil.convert_order_item_to_shipstation_order(order_item)
      if not ss_order:
        continue

      advanced_options = ShipStationAdvancedOptions(store_id=self.store_id)
      ss_order.advanced_options = advanced_options

      ss_orders[ss_order.order_number] = ss_order

    to_update = []
    to_create = []

    order_numbers = list(ss_orders.keys())
    for so in ShipstationOrder.select().where(ShipstationOrder.order_number.in_(order_numbers)).execute():
      ss_order = ss_orders.pop(so.order_number, None)
      if ss_order.order_status != 'cancelled':
        continue

      ss_order.order_key = so.order_key
      to_update.append(ss_order)

    for _, ss_order in ss_orders.items():
      to_create.append(ss_order)

    if to_update:
      chunks = [to_update[x:x + batch_size] for x in range(0, len(to_update), batch_size)]
      for chunk in chunks:
        for so in chunk:
          self.client.add_order(so)
        try:
          resp = self.client.submit_orders()
          self.client.clear_orders()
          self.save_shipstation_orders(resp)
        except Exception as e:
          logger.exception(e)

    if to_create:
      chunks = [to_create[x:x + batch_size] for x in range(0, len(to_create), batch_size)]
      for chunk in chunks:
        for so in chunk:
          self.client.add_order(so)
        try:
          resp = self.client.submit_orders()
          self.client.clear_orders()
          self.save_shipstation_orders(resp)
        except Exception as e:
          logger.exception(e)

  def load_orders_from_shipstation(self):
    pass

  def save_shipstation_orders(self, resp):
    sos = dict()
    res = resp.json()
    for result in res['results']:
      succeed = result.get('success', True)
      if not succeed:
        continue

      order_number = result['orderNumber']
      sos[order_number] = {
        'order_id': result['orderId'],
        'order_number': result['orderNumber'],
        'order_key': result['orderKey']
      }

    order_numbers = list(sos.keys())
    for so in ShipstationOrder.select().where(ShipstationOrder.order_number.in_(order_numbers)).execute():
      if so.order_number not in sos:
        continue

      sso = sos.pop(so.order_number)
      if so.order_id != sso['order_id'] or so.order_key != sso['order_key']:
        so.order_id = sso['order_id']
        so.order_key = sso['order_key']
        so.save()

    if sos:
      s_order_numbers = []
      sos_mapping = dict()
      for _, so in sos.items():
        parts = so['order_number'].split('-')
        s_line_item_id = parts.pop()
        s_order_number = '-'.join(parts)
        s_order_numbers.append(s_order_number)
      for oi in OrderItem.select().where(OrderItem.s_order_number.in_(s_order_numbers)).execute():
        k = '{}-{}'.format(oi.s_order_number, oi.s_line_item_id)
        if k in sos:
          sos[k]['order_item_id'] = oi.id

      to_create = []
      for so in sos.values():
        if 'order_item_id' not in so:
          continue

        to_create.append(so)

      ShipstationOrder.insert_many(to_create).execute()
